@@ -38,11 +38,14 @@ struct BuilderSourceArchive
 
   char         *path;
   char         *url;
+  char        **mirror_urls;
   char         *md5;
   char         *sha1;
   char         *sha256;
   char         *sha512;
   guint         strip_components;
+  char         *dest_filename;
+  gboolean      git_init;
 };
 
 typedef struct
@@ -61,6 +64,9 @@ enum {
   PROP_SHA256,
   PROP_SHA512,
   PROP_STRIP_COMPONENTS,
+  PROP_DEST_FILENAME,
+  PROP_MIRROR_URLS,
+  PROP_GIT_INIT,
   LAST_PROP
 };
 
@@ -127,6 +133,8 @@ builder_source_archive_finalize (GObject *object)
   g_free (self->sha1);
   g_free (self->sha256);
   g_free (self->sha512);
+  g_free (self->dest_filename);
+  g_strfreev (self->mirror_urls);
 
   G_OBJECT_CLASS (builder_source_archive_parent_class)->finalize (object);
 }
@@ -169,6 +177,18 @@ builder_source_archive_get_property (GObject    *object,
       g_value_set_uint (value, self->strip_components);
       break;
 
+    case PROP_DEST_FILENAME:
+      g_value_set_string (value, self->dest_filename);
+      break;
+
+    case PROP_MIRROR_URLS:
+      g_value_set_boxed (value, self->mirror_urls);
+      break;
+
+    case PROP_GIT_INIT:
+      g_value_set_boolean (value, self->git_init);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
     }
@@ -181,6 +201,7 @@ builder_source_archive_set_property (GObject      *object,
                                      GParamSpec   *pspec)
 {
   BuilderSourceArchive *self = BUILDER_SOURCE_ARCHIVE (object);
+  gchar **tmp;
 
   switch (prop_id)
     {
@@ -218,6 +239,21 @@ builder_source_archive_set_property (GObject      *object,
       self->strip_components = g_value_get_uint (value);
       break;
 
+    case PROP_DEST_FILENAME:
+      g_free (self->dest_filename);
+      self->dest_filename = g_value_dup_string (value);
+      break;
+
+    case PROP_MIRROR_URLS:
+      tmp = self->mirror_urls;
+      self->mirror_urls = g_strdupv (g_value_get_boxed (value));
+      g_strfreev (tmp);
+      break;
+
+    case PROP_GIT_INIT:
+      self->git_init = g_value_get_boolean (value);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
     }
@@ -263,7 +299,10 @@ get_download_location (BuilderSourceArchive *self,
 
   path = soup_uri_get_path (uri);
 
-  base_name = g_path_get_basename (path);
+  if (self->dest_filename)
+    base_name = g_strdup (self->dest_filename);
+  else
+    base_name = g_path_get_basename (path);
 
   builder_get_all_checksums (checksums, checksums_type,
                              self->md5,
@@ -358,21 +397,10 @@ builder_source_archive_download (BuilderSource  *source,
 
   if (g_file_query_exists (file, NULL))
     {
-      if (is_local && checksums[0]  != NULL)
-        {
-          g_autofree char *data = NULL;
-          gsize len;
-
-          if (!g_file_load_contents (file, NULL, &data, &len, NULL, error))
-            return FALSE;
-
-          if (!builder_verify_checksums (base_name,
-                                         data, len,
-                                         checksums, checksums_type,
-                                         error))
-            return FALSE;
-        }
-      return TRUE;
+      return !is_local || checksums[0] == NULL ||
+             builder_verify_checksums (base_name, file,
+                                       checksums, checksums_type,
+                                       error);
     }
 
   if (is_local)
@@ -383,6 +411,7 @@ builder_source_archive_download (BuilderSource  *source,
 
   if (!builder_context_download_uri (context,
                                      self->url,
+                                     (const char **)self->mirror_urls,
                                      file,
                                      checksums,
                                      checksums_type,
@@ -475,7 +504,8 @@ get_type (GFile *archivefile)
   if (g_str_has_suffix (lower, ".tar.lzo"))
     return TAR_LZOP;
 
-  if (g_str_has_suffix (lower, ".tar.xz"))
+  if (g_str_has_suffix (lower, ".tar.xz") ||
+      g_str_has_suffix (lower, ".txz"))
     return TAR_XZ;
 
   if (g_str_has_suffix (lower, ".zip"))
@@ -566,6 +596,39 @@ create_uncompress_directory (BuilderSourceArchive *self, GFile *dest, GError **e
   return uncompress_dest;
 }
 
+static gboolean
+git (GFile   *dir,
+     GError **error,
+     ...)
+{
+  gboolean res;
+  va_list ap;
+
+  va_start (ap, error);
+  res = flatpak_spawn (dir, NULL, 0, error, "git", ap);
+  va_end (ap);
+
+  return res;
+}
+
+static gboolean
+init_git (GFile   *dir,
+          GError **error)
+{
+  char *basename;
+
+  basename = g_file_get_basename (dir);
+  if (!git (dir, error, "init", NULL) ||
+      !git (dir, error, "add", "--ignore-errors", ".", NULL) ||
+      !git (dir, error, "commit", "-m", basename, NULL))
+    {
+      g_free (basename);
+      return FALSE;
+    }
+
+  return TRUE;
+}
+
 static gboolean
 builder_source_archive_extract (BuilderSource  *source,
                                 GFile          *dest,
@@ -635,6 +698,12 @@ builder_source_archive_extract (BuilderSource  *source,
       return FALSE;
     }
 
+  if (self->git_init)
+    {
+      if (!init_git (dest, error))
+        return FALSE;
+    }
+
   return TRUE;
 }
 
@@ -705,6 +774,8 @@ builder_source_archive_checksum (BuilderSource  *source,
   builder_cache_checksum_compat_str (cache, self->sha1);
   builder_cache_checksum_compat_str (cache, self->sha512);
   builder_cache_checksum_uint32 (cache, self->strip_components);
+  builder_cache_checksum_compat_str (cache, self->dest_filename);
+  builder_cache_checksum_compat_strv (cache, self->mirror_urls);
 }
 
 
@@ -774,6 +845,27 @@ builder_source_archive_class_init (BuilderSourceArchiveClass *klass)
                                                       0, G_MAXUINT,
                                                       1,
                                                       G_PARAM_READWRITE));
+  g_object_class_install_property (object_class,
+                                   PROP_DEST_FILENAME,
+                                   g_param_spec_string ("dest-filename",
+                                                        "",
+                                                        "",
+                                                        NULL,
+                                                        G_PARAM_READWRITE));
+  g_object_class_install_property (object_class,
+                                   PROP_MIRROR_URLS,
+                                   g_param_spec_boxed ("mirror-urls",
+                                                       "",
+                                                       "",
+                                                       G_TYPE_STRV,
+                                                       G_PARAM_READWRITE));
+  g_object_class_install_property (object_class,
+                                   PROP_GIT_INIT,
+                                   g_param_spec_boolean ("git-init",
+                                                         "",
+                                                         "",
+                                                         FALSE,
+                                                         G_PARAM_READWRITE));
 }
 
 static void
